@@ -16,17 +16,17 @@ import sys
 from concurrent.futures import ThreadPoolExecutor
 from contextvars import ContextVar
 from types import ModuleType
-from typing import Any, Callable, Dict, List, Optional, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 import unify
 from tqdm import tqdm
 from unify import BASE_URL
-from unify.utils import _requests
+from unify.utils import http
 from unify.utils.helpers import flexible_deepcopy
 
 from ...utils._caching import _get_cache, _write_to_cache, is_caching_enabled
 from ...utils.helpers import (
-    _check_response,
+    _create_request_header,
     _get_and_maybe_create_project,
     _validate_api_key,
 )
@@ -118,10 +118,7 @@ class _AsyncTraceLogger:
         atexit.register(self.shutdown, flush=True)
         signal.signal(signal.SIGINT, self._on_sigint)
 
-        headers = {
-            "accept": "application/json",
-            "Authorization": f"Bearer {self._api_key}",
-        }
+        headers = _create_request_header(self._api_key)
 
         self._loop = asyncio.new_event_loop()
         self._loop.set_default_executor(
@@ -439,47 +436,68 @@ def mark_spans_as_done(
         )
 
 
-def _apply_row_ids(
+def _apply_row_ids_and_non_unique_auto_count_vals(
     row_ids_data: Optional[Dict[str, Any]],
+    auto_counting_data: Optional[Dict[str, List[Any]]],
     entries: List[Dict[str, Any]],
 ) -> None:
     """
-    Apply row_ids data from server response to entry dictionaries.
+    Apply unique row_ids and non-unique auto_counting values from server response to entries.
 
-    Handles the new standardized format {'names': List[str], 'ids': List[List[int]]}
-    while maintaining backward compatibility with the old format during transition.
+    Behavior:
+    - Apply row_ids (unique auto-incrementing keys) first, using the standardized format
+      {'names': List[str], 'ids': List[List[int]]}.
+    - Then, apply auto_counting key/value pairs ONLY for keys not already included in
+      row_ids.names. This ensures unique keys come from row_ids while additional
+      non-unique counters (e.g., independent counters) are also populated.
 
     Args:
-        row_ids_data: The row_ids data from server response
-        entries: List of entry dictionaries to update with row_ids
+        row_ids_data: The row_ids data from server response.
+        auto_counting_data: The auto_counting data from server response, mapping key -> List[Any].
+        entries: List of entry dictionaries to update.
     """
-    if not row_ids_data:
-        return
+    # 1) Apply row_ids (unique keys)
+    row_id_names: List[str] = []
+    if row_ids_data:
+        names = row_ids_data.get("names")
+        ids = row_ids_data.get("ids")
 
-    names = row_ids_data.get("names")
-    ids = row_ids_data.get("ids")
+        if names and ids:
+            # Ensure names is always a list for consistent processing
+            if not isinstance(names, list):
+                names = [names]
+            row_id_names = names
 
-    if not names or not ids:
-        return
+            # Apply IDs to entries
+            for entry, id_values in zip(entries, ids):
+                if id_values is not None:
+                    # Handle both nested ID format (list of lists) and flat format (list of values)
+                    if isinstance(id_values, list) and len(names) > 1:
+                        # Nested format: zip names with id_values
+                        id_dict = dict(zip(names, id_values))
+                        entry.update(id_dict)
+                    else:
+                        # Single ID format: use first name with the id_value
+                        if isinstance(id_values, list) and len(id_values) == 1:
+                            entry[names[0]] = id_values[0]
+                        else:
+                            entry[names[0]] = id_values
 
-    # Ensure names is always treated as a list for consistent processing
-    if not isinstance(names, list):
-        names = [names]
-
-    # Apply IDs to entries
-    for entry, id_values in zip(entries, ids):
-        if id_values is not None:
-            # Handle both nested ID format (list of lists) and flat format (list of values)
-            if isinstance(id_values, list) and len(names) > 1:
-                # Nested format: zip names with id_values
-                id_dict = dict(zip(names, id_values))
-                entry.update(id_dict)
-            else:
-                # Single ID format: use first name with the id_value
-                if isinstance(id_values, list) and len(id_values) == 1:
-                    entry[names[0]] = id_values[0]
-                else:
-                    entry[names[0]] = id_values
+    # 2) Apply auto_counting (non-unique and any additional counters)
+    if auto_counting_data:
+        # Treat row_id_names as a set for fast membership checks
+        row_id_names_set = set(row_id_names)
+        # For each auto_counting key, if not a row_id key, propagate values into entries
+        for key, values in auto_counting_data.items():
+            if key in row_id_names_set:
+                # Skip keys already applied via row_ids
+                continue
+            if not isinstance(values, list):
+                # Defensive: backend guarantees list, but guard just in case
+                continue
+            for idx, entry in enumerate(entries):
+                if idx < len(values):
+                    entry[key] = values[idx]
 
 
 def _handle_cache(fn: Callable) -> Callable:
@@ -597,15 +615,30 @@ def _handle_mutability(
         single_item = True
         new_data = [flexible_deepcopy(data, on_fail="shallow")]
     if isinstance(mutable, dict):
-        for field, mut in mutable.items():
-            for item in new_data:
+        for item in new_data:
+            et = item.get("explicit_types")
+            if not isinstance(et, dict):
+                et = {}
+                item["explicit_types"] = et
+            for field, mut in mutable.items():
                 if field in item:
-                    item.setdefault("explicit_types", {})[field] = {"mutable": mut}
+                    existing = (
+                        et.get(field, {}) if isinstance(et.get(field, {}), dict) else {}
+                    )
+                    existing = {**existing, "mutable": mut}
+                    et[field] = existing
     elif isinstance(mutable, bool):
         for item in new_data:
+            et = item.get("explicit_types")
+            if not isinstance(et, dict):
+                et = {}
+                item["explicit_types"] = et
             for k in list(item.keys()):
-                if k != "explicit_types":
-                    item.setdefault("explicit_types", {})[k] = {"mutable": mutable}
+                if k == "explicit_types":
+                    continue
+                existing = et.get(k, {}) if isinstance(et.get(k, {}), dict) else {}
+                existing = {**existing, "mutable": mutable}
+                et[k] = existing
     if single_item:
         return new_data[0]
     return new_data
@@ -775,10 +808,7 @@ def _sync_log(
 
     This is a helper function used when async logging is disabled or unavailable.
     """
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
 
     body = {
         "project": project,
@@ -786,12 +816,15 @@ def _sync_log(
         "params": params,
         "entries": entries,
     }
-    response = _requests.post(BASE_URL + "/logs", headers=headers, json=body)
-    _check_response(response)
+    response = http.post(BASE_URL + "/logs", headers=headers, json=body)
     resp_json = response.json()
 
-    # Apply row_ids to entries using the centralized helper
-    _apply_row_ids(resp_json.get("row_ids"), [entries])
+    # Apply row_ids and non-unique auto_counting values to entries
+    _apply_row_ids_and_non_unique_auto_count_vals(
+        resp_json.get("row_ids"),
+        resp_json.get("auto_counting"),
+        [entries],
+    )
 
     return unify.Log(
         id=resp_json["log_event_ids"][0],
@@ -924,10 +957,7 @@ def create_logs(
     api_key = _validate_api_key(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = _handle_context(context)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     # ToDo: add support for all of the context variables, as is done for `unify.log` above
     params = _handle_mutability(mutable, params)
     entries = _handle_mutability(mutable, entries)
@@ -946,18 +976,21 @@ def create_logs(
         batched = body_size < CHUNK_LIMIT
     if batched:
         if body_size < CHUNK_LIMIT:
-            response = _requests.post(BASE_URL + "/logs", headers=headers, json=body)
+            response = http.post(BASE_URL + "/logs", headers=headers, json=body)
         else:
-            response = _requests.post(
+            response = http.post(
                 BASE_URL + "/logs",
                 headers=headers,
                 data=_json_chunker(body),
             )
-        _check_response(response)
         resp_json = response.json()
 
-        # Apply row_ids to entries using the centralized helper
-        _apply_row_ids(resp_json.get("row_ids"), entries)
+        # Apply row_ids and non-unique auto_counting values to entries
+        _apply_row_ids_and_non_unique_auto_count_vals(
+            resp_json.get("row_ids"),
+            resp_json.get("auto_counting"),
+            entries,
+        )
 
         return [
             unify.Log(
@@ -1045,10 +1078,7 @@ def _add_to_log(
     else:
         # Fallback to synchronous update if async logging isn’t enabled.
         log_ids = _to_log_ids(logs)
-        headers = {
-            "accept": "application/json",
-            "Authorization": f"Bearer {api_key}",
-        }
+        headers = _create_request_header(api_key)
         all_kwargs = []
         if nest_level.get() > 0:
             for log_id in log_ids:
@@ -1066,8 +1096,7 @@ def _add_to_log(
             ), "All logs must share the same context if they're all being updated at the same time."
             data = all_kwargs[0]
         body = {"logs": log_ids, mode: data, "overwrite": overwrite, "context": context}
-        response = _requests.put(BASE_URL + "/logs", headers=headers, json=body)
-        _check_response(response)
+        response = http.put(BASE_URL + "/logs", headers=headers, json=body)
         if nest_level.get() > 0:
             logged = LOGGED.get()
             new_logged = {}
@@ -1181,11 +1210,7 @@ def update_logs(
     """
     if not logs and not params and not entries:
         return {"detail": "No logs to update."}
-    api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     log_ids = _to_log_ids(logs)
     body = {
         "logs": log_ids,
@@ -1196,8 +1221,7 @@ def update_logs(
         body["entries"] = entries
     if params is not None:
         body["params"] = params
-    response = _requests.put(BASE_URL + "/logs", headers=headers, json=body)
-    _check_response(response)
+    response = http.put(BASE_URL + "/logs", headers=headers, json=body)
     return response.json()
 
 
@@ -1238,11 +1262,7 @@ def delete_logs(
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = context if context else CONTEXT_READ.get()
     log_ids = _to_log_ids(logs)
-    api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     body = {
         "project": project,
         "context": context,
@@ -1250,13 +1270,12 @@ def delete_logs(
         "source_type": source_type,
     }
     params = {"delete_empty_logs": delete_empty_logs}
-    response = _requests.delete(
+    response = http.delete(
         BASE_URL + "/logs",
         headers=headers,
         params=params,
         json=body,
     )
-    _check_response(response)
     if USR_LOGGING:
         logger.info(f"Deleted Logs({', '.join([str(i) for i in log_ids])})")
     return response.json()
@@ -1290,10 +1309,7 @@ def delete_log_fields(
     """
     log_ids = _to_log_ids(logs)
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = context if context else CONTEXT_READ.get()
     body = {
@@ -1301,12 +1317,11 @@ def delete_log_fields(
         "context": context,
         "ids_and_fields": [(log_ids, field)],
     }
-    response = _requests.delete(
+    response = http.delete(
         BASE_URL + f"/logs",
         headers=headers,
         json=body,
     )
-    _check_response(response)
     if USR_LOGGING:
         logger.info(
             f"Deleted Field `{field}` from Logs({', '.join([str(i) for i in log_ids])})",
@@ -1401,10 +1416,7 @@ def get_logs(
     """
     # ToDo: add support for all context handlers
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(
         project,
         api_key=api_key,
@@ -1447,8 +1459,7 @@ def get_logs(
         "return_timestamps": return_timestamps,
     }
 
-    response = _requests.get(BASE_URL + "/logs", headers=headers, params=params)
-    _check_response(response)
+    response = http.get(BASE_URL + "/logs", headers=headers, params=params)
 
     if not group_by:
         if return_ids_only:
@@ -1486,17 +1497,13 @@ def get_log_by_id(
         The full set of log data.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
-    response = _requests.get(
+    response = http.get(
         BASE_URL + "/logs",
         params={"project": project, "from_ids": [id]},
         headers=headers,
     )
-    _check_response(response)
     params, lgs, count = response.json().values()
     if len(lgs) == 0:
         raise Exception(f"Log with id {id} does not exist")
@@ -1515,62 +1522,136 @@ def get_log_by_id(
 def get_logs_metric(
     *,
     metric: str,
-    key: str,
-    filter: Optional[str] = None,
+    key: Union[str, List[str]],
+    filter: Optional[Union[str, Dict[str, str]]] = None,
     project: Optional[str] = None,
     context: Optional[str] = None,
-    from_ids: Optional[List[int]] = None,
-    exclude_ids: Optional[List[int]] = None,
+    from_ids: Optional[Union[List[int], Dict[str, str]]] = None,
+    exclude_ids: Optional[Union[List[int], Dict[str, str]]] = None,
+    group_by: Optional[Union[str, List[str]]] = None,
     api_key: Optional[str] = None,
-) -> Union[float, int, bool]:
+) -> Union[float, int, bool, str, Dict[str, Any]]:
     """
     Retrieve a set of log metrics across a project, after applying the filtering.
 
+    This endpoint supports three modes of operation:
+
+    1. Single key, no grouping: Returns a single metric value
+       Example:
+       get_logs_metric(metric="mean", key="score")
+       Response: 4.56
+
+    2. Multiple keys, no grouping: Returns a dict mapping keys to metric values
+       Example:
+       get_logs_metric(metric="mean", key=["score", "length"])
+       Response: {"score": 4.56, "length": 120}
+
+    3. With grouping: Returns metrics grouped by one or more fields
+       Example:
+       get_logs_metric(metric="mean", key="score", group_by="model")
+       Response: {"gpt-4": 4.56, "gpt-3.5": 3.78}
+
+       For nested grouping, provide a list of fields:
+       Example:
+       get_logs_metric(metric="mean", key="score", group_by=["model", "temperature"])
+       Response: {"gpt-4": {"0.7": 4.56, "0.9": 4.23}, "gpt-3.5": {"0.7": 3.78, "0.9": 3.45}}
+
     Args:
-        metric: The reduction metric to compute for the specified key. Supported are:
-        sum, mean, var, std, min, max, median, mode.
+        metric: The reduction metric to compute for the specified key(s). Supported are:
+            sum, mean, var, std, min, max, median, mode.
 
-        key: The key to compute the reduction statistic for.
+        key: The key(s) to compute the reduction statistic for. Can be a single string
+            for one key, or a list of strings for multiple keys.
 
-        filter: The filtering to apply to the various log values, expressed as a string,
-        for example:
-        "(temperature > 0.5 and (len(system_msg) < 100 or 'no' in usr_response))"
+        filter: The filtering to apply to the log values. Can be:
+            - A single string expression for all keys, e.g.:
+              "(temperature > 0.5 and (len(system_msg) < 100 or 'no' in usr_response))"
+            - A dict mapping keys to filter expressions for key-specific filtering, e.g.:
+              {"score": "score > 0", "length": "length < 100"}
 
         project: The id of the project to retrieve the logs for.
 
         context: The context of the logs to retrieve the metrics for.
 
-        from_ids: A list of log IDs to include in the results.
+        from_ids: Log IDs to include in the results. Can be:
+            - A list of integers for all keys, e.g.: [1, 2, 3]
+            - A dict mapping keys to ID strings for key-specific filtering, e.g.:
+              {"score": "1&2", "length": "3&4"}
 
-        exclude_ids: A list of log IDs to exclude from the results.
+        exclude_ids: Log IDs to exclude from the results. Can be:
+            - A list of integers for all keys, e.g.: [1, 2, 3]
+            - A dict mapping keys to ID strings for key-specific filtering, e.g.:
+              {"score": "1&2", "length": "3&4"}
+
+        group_by: Field(s) to group the metrics by. Can be:
+            - A single string for single-level grouping, e.g.: "model"
+            - A list of strings for nested grouping, e.g.: ["model", "temperature"]
+            Each field can be prefixed with "params/" to indicate it's a parameter,
+            or "entries/" or "derived_entries/" for entry fields.
 
         api_key: If specified, unify API key to be used. Defaults to the value in the
-        `UNIFY_KEY` environment variable.
+            `UNIFY_KEY` environment variable.
 
     Returns:
-        The full set of reduced log metrics for the project, after optionally applying
-        the optional filtering.
+        The metric value(s) for the project, after optionally applying filtering and grouping.
+        Return type depends on the mode:
+        - Single key, no grouping: scalar (float, int, bool, or str)
+        - Multiple keys, no grouping: dict mapping keys to scalar values
+        - With grouping: dict with nested structure based on grouping levels
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
+
+    # Build params dict
     params = {
         "project": project,
-        "filter_expr": filter,
-        "key": key,
-        "from_ids": "&".join(map(str, from_ids)) if from_ids else None,
-        "exclude_ids": "&".join(map(str, exclude_ids)) if exclude_ids else None,
         "context": context if context else CONTEXT_READ.get(),
     }
-    response = _requests.get(
+
+    # Handle key parameter - JSON encode if it's a list
+    if isinstance(key, list):
+        params["key"] = json.dumps(key)
+    else:
+        params["key"] = key
+
+    # Handle filter_expr parameter - JSON encode if it's a dict
+    if filter is not None:
+        if isinstance(filter, dict):
+            params["filter_expr"] = json.dumps(filter)
+        else:
+            params["filter_expr"] = filter
+
+    # Handle from_ids parameter
+    if from_ids is not None:
+        if isinstance(from_ids, dict):
+            # Key-specific from_ids - JSON encode the dict
+            params["from_ids"] = json.dumps(from_ids)
+        else:
+            # Legacy format - join list with &
+            params["from_ids"] = "&".join(map(str, from_ids))
+
+    # Handle exclude_ids parameter
+    if exclude_ids is not None:
+        if isinstance(exclude_ids, dict):
+            # Key-specific exclude_ids - JSON encode the dict
+            params["exclude_ids"] = json.dumps(exclude_ids)
+        else:
+            # Legacy format - join list with &
+            params["exclude_ids"] = "&".join(map(str, exclude_ids))
+
+    # Handle group_by parameter - JSON encode if it's a list
+    if group_by is not None:
+        if isinstance(group_by, list):
+            params["group_by"] = json.dumps(group_by)
+        else:
+            params["group_by"] = group_by
+
+    response = http.get(
         BASE_URL + f"/logs/metric/{metric}",
         headers=headers,
         params=params,
     )
-    _check_response(response)
     return response.json()
 
 
@@ -1578,6 +1659,7 @@ def get_groups(
     *,
     key: str,
     project: Optional[str] = None,
+    context: Optional[str] = None,
     filter: Optional[Dict[str, Any]] = None,
     from_ids: Optional[List[int]] = None,
     exclude_ids: Optional[List[int]] = None,
@@ -1592,6 +1674,8 @@ def get_groups(
 
         project: Name of the project to get logs from.
 
+        context: The context to get groups from.
+
         filter: Boolean string to filter logs, for example:
         "(temperature > 0.5 and (len(system_msg) < 100 or 'no' in usr_response))"
 
@@ -1607,20 +1691,18 @@ def get_groups(
         version of the log key with equal values, and the value being the equal value.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
+    context = context if context else CONTEXT_READ.get()
     params = {
         "project": project,
+        "context": context,
         "key": key,
         "filter_expr": filter,
         "from_ids": from_ids,
         "exclude_ids": exclude_ids,
     }
-    response = _requests.get(BASE_URL + "/logs/groups", headers=headers, params=params)
-    _check_response(response)
+    response = http.get(BASE_URL + "/logs/groups", headers=headers, params=params)
     return response.json()
 
 
@@ -1641,10 +1723,7 @@ def get_logs_latest_timestamp(
     Returns the update timestamp of the most recently updated log within the specified page and filter bounds.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = context if context else CONTEXT_READ.get()
     column_context = column_context if column_context else COLUMN_CONTEXT_READ.get()
@@ -1659,12 +1738,11 @@ def get_logs_latest_timestamp(
         "limit": limit,
         "offset": offset,
     }
-    response = _requests.get(
+    response = http.get(
         BASE_URL + "/logs/latest_timestamp",
         headers=headers,
         params=params,
     )
-    _check_response(response)
     return response.json()
 
 
@@ -1701,10 +1779,7 @@ def update_derived_log(
         A message indicating whether the derived logs were successfully updated.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = context if context else CONTEXT_WRITE.get()
     body = {
@@ -1715,17 +1790,57 @@ def update_derived_log(
         "equation": equation,
         "referenced_logs": referenced_logs,
     }
-    response = _requests.put(BASE_URL + "/logs/derived", headers=headers, json=body)
-    _check_response(response)
+    response = http.put(BASE_URL + "/logs/derived", headers=headers, json=body)
+    return response.json()
+
+
+def create_derived_logs(
+    *,
+    key: str,
+    equation: str,
+    referenced_logs,
+    derived: Optional[bool] = None,
+    project: Optional[str] = None,
+    context: Optional[str] = None,
+    api_key: Optional[str] = None,
+) -> Dict[str, Any]:
+    """
+    Creates one or more entries based on equation and referenced_logs.
+
+    Args:
+        key: The name of the entry.
+        equation: The equation for computing the value of each derived entry.
+        referenced_logs: The logs to use for each newly created derived entry,
+        either as a list of log ids or as a set of arguments for the get_logs endpoint.
+        derived: Whether to create derived logs (True) or static entries in base logs (False).
+
+    Returns:
+        A message indicating whether the derived logs were successfully created.
+    """
+    api_key = _validate_api_key(api_key)
+    headers = _create_request_header(api_key)
+    project = _get_and_maybe_create_project(project, api_key=api_key)
+    context = context if context else CONTEXT_WRITE.get()
+    body = {
+        "project": project,
+        "context": context,
+        "key": key,
+        "equation": equation,
+        "referenced_logs": referenced_logs,
+    }
+    if derived is not None:
+        body["derived"] = derived
+    response = http.post(BASE_URL + "/logs/derived", headers=headers, json=body)
     return response.json()
 
 
 def join_logs(
     *,
-    pair_of_args: List[Dict[str, Any]],
+    pair_of_args: Tuple[Dict[str, Any], Dict[str, Any]],
     join_expr: str,
     mode: str,
     new_context: str,
+    copy: Optional[bool] = True,
     columns: Optional[List[str]] = None,
     project: Optional[str] = None,
     api_key: Optional[str] = None,
@@ -1734,27 +1849,25 @@ def join_logs(
     Join two sets of logs based on specified criteria and creates new logs with the joined data.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
-    params = {
+    body = {
         "project": project,
         "pair_of_args": pair_of_args,
         "join_expr": join_expr,
         "mode": mode,
         "new_context": new_context,
         "columns": columns,
+        "copy": copy,
     }
-    response = _requests.post(BASE_URL + "/logs/join", headers=headers, params=params)
-    _check_response(response)
+    response = http.post(BASE_URL + "/logs/join", headers=headers, json=body)
     return response.json()
 
 
 def create_fields(
     fields: Union[Dict[str, Any], List[str]],
     *,
+    backfill_logs: Optional[bool] = None,
     project: Optional[str] = None,
     context: Optional[str] = None,
     api_key: Optional[str] = None,
@@ -1773,10 +1886,7 @@ def create_fields(
         `UNIFY_KEY` environment variable.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = context if context else CONTEXT_WRITE.get()
     if isinstance(fields, list):
@@ -1786,8 +1896,9 @@ def create_fields(
         "context": context,
         "fields": fields,
     }
-    response = _requests.post(BASE_URL + "/logs/fields", headers=headers, json=body)
-    _check_response(response)
+    if backfill_logs is not None:
+        body["backfill_logs"] = backfill_logs
+    response = http.post(BASE_URL + "/logs/fields", headers=headers, json=body)
     return response.json()
 
 
@@ -1815,10 +1926,7 @@ def rename_field(
         `UNIFY_KEY` environment variable.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = context if context else CONTEXT_WRITE.get()
     body = {
@@ -1827,12 +1935,11 @@ def rename_field(
         "old_field_name": name,
         "new_field_name": new_name,
     }
-    response = _requests.patch(
+    response = http.patch(
         BASE_URL + "/logs/rename_field",
         headers=headers,
         json=body,
     )
-    _check_response(response)
     return response.json()
 
 
@@ -1857,18 +1964,14 @@ def get_fields(
         A dictionary of fields names and their types
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = context if context else CONTEXT_READ.get()
     params = {
         "project": project,
         "context": context,
     }
-    response = _requests.get(BASE_URL + "/logs/fields", headers=headers, params=params)
-    _check_response(response)
+    response = http.get(BASE_URL + "/logs/fields", headers=headers, params=params)
     return response.json()
 
 
@@ -1893,10 +1996,7 @@ def delete_fields(
         `UNIFY_KEY` environment variable.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     project = _get_and_maybe_create_project(project, api_key=api_key)
     context = context if context else CONTEXT_WRITE.get()
     body = {
@@ -1904,12 +2004,11 @@ def delete_fields(
         "context": context,
         "fields": fields,
     }
-    response = _requests.delete(
+    response = http.delete(
         BASE_URL + "/logs/fields",
         headers=headers,
         json=body,
     )
-    _check_response(response)
     return response.json()
 
 
