@@ -1,9 +1,9 @@
 from typing import Dict, List, Optional, Union
 
 from unify import BASE_URL
-from unify.utils import _requests
+from unify.utils import http
 
-from ...utils.helpers import _check_response, _validate_api_key
+from ...utils.helpers import _create_request_header, _validate_api_key
 
 # Projects #
 # ---------#
@@ -38,10 +38,7 @@ def create_project(
         A message indicating whether the project was created successfully.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     body = {"name": name, "is_versioned": is_versioned}
     if overwrite:
         if name in list_projects(api_key=api_key):
@@ -51,8 +48,7 @@ def create_project(
                 return delete_project_contexts(name=name, api_key=api_key)
             else:
                 delete_project(name=name, api_key=api_key)
-    response = _requests.post(BASE_URL + "/project", headers=headers, json=body)
-    _check_response(response)
+    response = http.post(BASE_URL + "/project", headers=headers, json=body)
     return response.json()
 
 
@@ -77,17 +73,13 @@ def rename_project(
         A message indicating whether the project was successfully renamed.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     body = {"name": new_name}
-    response = _requests.patch(
+    response = http.patch(
         BASE_URL + f"/project/{name}",
         headers=headers,
         json=body,
     )
-    _check_response(response)
     return response.json()
 
 
@@ -109,12 +101,8 @@ def delete_project(
         Whether the project was successfully deleted.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
-    response = _requests.delete(BASE_URL + f"/project/{name}", headers=headers)
-    _check_response(response)
+    headers = _create_request_header(api_key)
+    response = http.delete(BASE_URL + f"/project/{name}", headers=headers)
     return response.json()
 
 
@@ -133,12 +121,8 @@ def delete_project_logs(
         `UNIFY_KEY` environment variable.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
-    response = _requests.delete(BASE_URL + f"/project/{name}/logs", headers=headers)
-    _check_response(response)
+    headers = _create_request_header(api_key)
+    response = http.delete(BASE_URL + f"/project/{name}/logs", headers=headers)
     return response.json()
 
 
@@ -157,12 +141,8 @@ def delete_project_contexts(
         `UNIFY_KEY` environment variable.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
-    response = _requests.delete(BASE_URL + f"/project/{name}/contexts", headers=headers)
-    _check_response(response)
+    headers = _create_request_header(api_key)
+    response = http.delete(BASE_URL + f"/project/{name}/contexts", headers=headers)
     return response.json()
 
 
@@ -181,12 +161,8 @@ def list_projects(
         List of all project names.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
-    response = _requests.get(BASE_URL + "/projects", headers=headers)
-    _check_response(response)
+    headers = _create_request_header(api_key)
+    response = http.get(BASE_URL + "/projects", headers=headers)
     return response.json()
 
 
@@ -209,17 +185,13 @@ def commit_project(
         A dictionary containing the new commit_hash.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     body = {"commit_message": commit_message}
-    response = _requests.post(
+    response = http.post(
         BASE_URL + f"/project/{name}/commit",
         headers=headers,
         json=body,
     )
-    _check_response(response)
     return response.json()
 
 
@@ -242,17 +214,13 @@ def rollback_project(
         A message indicating the success of the rollback operation.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     body = {"commit_hash": commit_hash}
-    response = _requests.post(
+    response = http.post(
         BASE_URL + f"/project/{name}/rollback",
         headers=headers,
         json=body,
     )
-    _check_response(response)
     return response.json()
 
 
@@ -269,10 +237,6 @@ def get_project_commits(name: str, *, api_key: Optional[str] = None) -> List[Dic
         A list of dictionaries, each representing a commit.
     """
     api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
-    response = _requests.get(BASE_URL + f"/project/{name}/commits", headers=headers)
-    _check_response(response)
+    headers = _create_request_header(api_key)
+    response = http.get(BASE_URL + f"/project/{name}/commits", headers=headers)
     return response.json()
