@@ -1,5 +1,5 @@
 from . import base
-from .base import _Client
+from .base import _Client, set_client_direct_mode
 from . import uni_llm
 from .uni_llm import _UniClient, Unify, AsyncUnify
 from . import multi_llm
