@@ -18,15 +18,31 @@ from typing_extensions import Self
 
 # local
 from unify import BASE_URL
-from unify.utils import _requests
+from unify.utils import http
 
 # noinspection PyProtectedMember
-from unify.utils.helpers import _validate_api_key
+from unify.utils.helpers import (
+    _create_request_header,
+    _validate_api_key,
+    _validate_openai_api_key,
+)
+
+
+def set_client_direct_mode(value: bool) -> None:
+    """
+    Set the direct mode for the client.
+
+    Args:
+        value: The value to set the direct mode to.
+    """
+    _Client._set_direct_mode(value)
 
 
 class _Client(ABC):
     """Base Abstract class for interacting with the Unify chat completions endpoint."""
 
+    _DIRECT_OPENAI_MODE = False
+
     def __init__(
         self,
         *,
@@ -64,6 +80,7 @@ class _Client(ABC):
         log_query_body: Optional[bool],
         log_response_body: Optional[bool],
         api_key: Optional[str],
+        openai_api_key: Optional[str],
         # python client arguments
         stateful: bool,
         return_full_completion: bool,
@@ -78,6 +95,10 @@ class _Client(ABC):
 
         # initial values
         self._api_key = _validate_api_key(api_key)
+        self._openai_api_key = _validate_openai_api_key(
+            _Client._DIRECT_OPENAI_MODE,
+            openai_api_key,
+        )
         self._system_message = None
         self._messages = None
         self._frequency_penalty = None
@@ -179,6 +200,13 @@ class _Client(ABC):
             "reasoning_effort": reasoning_effort,
         }
 
+    @classmethod
+    def _set_direct_mode(cls, value: bool) -> None:
+        cls._DIRECT_OPENAI_MODE = value
+
+    def _is_direct_mode_available(self) -> bool:
+        return self._openai_api_key is not None and self._DIRECT_OPENAI_MODE
+
     # Properties #
     # -----------#
 
@@ -1276,12 +1304,9 @@ class _Client(ABC):
             ValueError: If there was an error parsing the JSON response.
         """
         url = f"{BASE_URL}/credits"
-        headers = {
-            "accept": "application/json",
-            "Authorization": f"Bearer {self._api_key}",
-        }
+        headers = _create_request_header(self._api_key)
         try:
-            response = _requests.get(url, headers=headers, timeout=10)
+            response = http.get(url, headers=headers, timeout=10)
             if response.status_code != 200:
                 raise Exception(response.json())
             return response.json()["credits"]
