@@ -20,10 +20,15 @@ from openai.types.chat import (
 from pydantic import BaseModel
 from typing_extensions import Self
 from unify import BASE_URL
-from unify.utils import _requests
+from unify.utils import http
 
 # noinspection PyProtectedMember
-from unify.utils.helpers import _default, _validate_api_key
+from unify.utils.helpers import (
+    _create_request_header,
+    _default,
+    _validate_api_key,
+    _validate_openai_api_key,
+)
 
 from ..clients import AsyncUnify, _Client, _UniClient
 
@@ -65,6 +70,7 @@ class _MultiClient(_Client, abc.ABC):
         log_query_body: Optional[bool] = True,
         log_response_body: Optional[bool] = True,
         api_key: Optional[str] = None,
+        openai_api_key: Optional[str] = None,
         # python client arguments
         stateful: bool = False,
         return_full_completion: bool = False,
@@ -250,6 +256,7 @@ class _MultiClient(_Client, abc.ABC):
             log_query_body=log_query_body,
             log_response_body=log_response_body,
             api_key=api_key,
+            openai_api_key=openai_api_key,
             # python client arguments
             stateful=stateful,
             return_full_completion=return_full_completion,
@@ -271,6 +278,10 @@ class _MultiClient(_Client, abc.ABC):
         else:
             endpoints = list(endpoints)
         self._api_key = _validate_api_key(api_key)
+        self._openai_api_key = _validate_openai_api_key(
+            self._DIRECT_OPENAI_MODE,
+            openai_api_key,
+        )
         self._endpoints = endpoints
         self._client_class = AsyncUnify
         self._clients = self._create_clients(endpoints)
@@ -306,6 +317,7 @@ class _MultiClient(_Client, abc.ABC):
                 log_query_body=self.log_query_body,
                 log_response_body=self.log_response_body,
                 api_key=self._api_key,
+                openai_api_key=self._openai_api_key,
                 # python client arguments
                 stateful=self.stateful,
                 return_full_completion=self.return_full_completion,
@@ -407,12 +419,9 @@ class _MultiClient(_Client, abc.ABC):
             ValueError: If there was an error parsing the JSON response.
         """
         url = f"{BASE_URL}/credits"
-        headers = {
-            "accept": "application/json",
-            "Authorization": f"Bearer {self._api_key}",
-        }
+        headers = _create_request_header(self._api_key)
         try:
-            response = _requests.get(url, headers=headers, timeout=10)
+            response = http.get(url, headers=headers, timeout=10)
             if response.status_code != 200:
                 raise Exception(response.json())
             return response.json()["credits"]
