@@ -1,5 +1,6 @@
 # global
 import abc
+import asyncio
 import inspect
 import threading
 
@@ -36,7 +37,7 @@ from openai.types.chat import (
 from openai.types.chat.chat_completion import Choice
 from pydantic import BaseModel
 from typing_extensions import Self
-from unify import BASE_URL, LOCAL_MODELS
+from unify import BASE_URL, LOCAL_MODELS, list_models
 from unify.universal_api.clients.helpers import (
     _assert_is_valid_endpoint,
     _assert_is_valid_model,
@@ -89,6 +90,7 @@ class _UniClient(_Client, abc.ABC):
         log_query_body: Optional[bool] = True,
         log_response_body: Optional[bool] = True,
         api_key: Optional[str] = None,
+        openai_api_key: Optional[str] = None,
         # python client arguments
         stateful: bool = False,
         return_full_completion: bool = False,
@@ -285,6 +287,7 @@ class _UniClient(_Client, abc.ABC):
             log_query_body=log_query_body,
             log_response_body=log_response_body,
             api_key=api_key,
+            openai_api_key=openai_api_key,
             # python client arguments
             stateful=stateful,
             return_full_completion=return_full_completion,
@@ -308,7 +311,6 @@ class _UniClient(_Client, abc.ABC):
                 "if the model or provider are passed, then the endpoint must not be"
                 "passed.",
             )
-        self._client = self._get_client()
         self._endpoint = None
         self._provider = None
         self._model = None
@@ -319,6 +321,16 @@ class _UniClient(_Client, abc.ABC):
         if model:
             self.set_model(model)
 
+        self._should_use_direct_mode = False
+        if self._is_direct_mode_available() and self.model in list_models(
+            provider="openai",
+        ):
+            self._should_use_direct_mode = True
+            self._endpoint = self.model
+            self._provider = "openai"
+
+        self._client = self._get_client()
+
     # Settable Properties #
     # --------------------#
 
@@ -957,6 +969,11 @@ class Unify(_UniClient):
 
     def _get_client(self):
         try:
+            if self._should_use_direct_mode:
+                return openai.OpenAI(
+                    api_key=self._openai_api_key,
+                    timeout=3600.0,  # one hour
+                )
             http_client = make_httpx_client_for_unify_logging(BASE_URL)
             return openai.OpenAI(
                 base_url=f"{BASE_URL}",
@@ -995,6 +1012,8 @@ class Unify(_UniClient):
             log_query_body=log_query_body,
             log_response_body=log_response_body,
         )
+        if self._should_use_direct_mode:
+            kw.pop("extra_body")
         try:
             if endpoint in LOCAL_MODELS:
                 kw.pop("extra_body")
@@ -1056,6 +1075,8 @@ class Unify(_UniClient):
             log_query_body=log_query_body,
             log_response_body=log_response_body,
         )
+        if self._should_use_direct_mode:
+            kw.pop("extra_body")
         if isinstance(cache, str) and cache.endswith("-closest"):
             cache = cache.removesuffix("-closest")
             read_closest = True
@@ -1145,6 +1166,16 @@ class Unify(_UniClient):
                     response=chat_completion,
                     backend=cache_backend,
                 )
+        if self._should_use_direct_mode and not in_cache:
+            response_format = kw.get("response_format")
+            if response_format is not None:
+                kw["response_format"] = response_format.model_json_schema()
+            unify.log_query(
+                endpoint=f"{endpoint}@openai",
+                query_body=kw,
+                response_body=chat_completion.model_dump(),
+                consume_credits=True,
+            )
         if return_full_completion:
             if endpoint == "user-input":
                 input_msg = sum(len(msg) for msg in prompt.components["messages"])
@@ -1291,6 +1322,12 @@ class AsyncUnify(_UniClient):
         try:
             # Async event hooks must use AsyncClient
             http_client = make_async_httpx_client_for_unify_logging(BASE_URL)
+            if self._should_use_direct_mode:
+                return openai.AsyncOpenAI(
+                    api_key=self._openai_api_key,
+                    timeout=3600.0,  # one hour
+                    http_client=http_client,
+                )
             return openai.AsyncOpenAI(
                 base_url=f"{BASE_URL}",
                 api_key=self._api_key,
@@ -1328,6 +1365,8 @@ class AsyncUnify(_UniClient):
             log_query_body=log_query_body,
             log_response_body=log_response_body,
         )
+        if self._should_use_direct_mode:
+            kw.pop("extra_body")
         try:
             if endpoint in LOCAL_MODELS:
                 kw.pop("extra_body")
@@ -1388,6 +1427,8 @@ class AsyncUnify(_UniClient):
             log_query_body=log_query_body,
             log_response_body=log_response_body,
         )
+        if self._should_use_direct_mode:
+            kw.pop("extra_body")
         if isinstance(cache, str) and cache.endswith("-closest"):
             cache = cache.removesuffix("-closest")
             read_closest = True
@@ -1481,6 +1522,20 @@ class AsyncUnify(_UniClient):
                     response=chat_completion,
                     backend=cache_backend,
                 )
+        if self._should_use_direct_mode and not in_cache:
+            response_format = kw.get("response_format")
+            if response_format is not None:
+                kw["response_format"] = response_format.model_json_schema()
+            asyncio.create_task(
+                asyncio.to_thread(
+                    unify.log_query,
+                    endpoint=f"{endpoint}@openai",
+                    query_body=kw,
+                    response_body=chat_completion.model_dump(),
+                    consume_credits=True,
+                ),
+                name="unify_client_log_query",
+            )
         if return_full_completion:
             return chat_completion
         content = chat_completion.choices[0].message.content
