@@ -2,9 +2,9 @@ import functools
 from typing import List, Optional
 
 from unify import BASE_URL
-from unify.utils import _requests
+from unify.utils import http
 
-from ...utils.helpers import _res_to_list, _validate_api_key
+from ...utils.helpers import _create_request_header, _res_to_list
 
 
 @functools.lru_cache(maxsize=32)
@@ -28,17 +28,13 @@ def list_providers(
         BadRequestError: If there was an HTTP error.
         ValueError: If there was an error parsing the JSON response.
     """
-    api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     url = f"{BASE_URL}/providers"
     if model:
         kw = dict(headers=headers, params={"model": model})
     else:
         kw = dict(headers=headers)
-    response = _requests.get(url, **kw)
+    response = http.get(url, **kw)
     if response.status_code != 200:
         raise Exception(response.json())
     return _res_to_list(response)
@@ -64,17 +60,13 @@ def list_models(
         BadRequestError: If there was an HTTP error.
         ValueError: If there was an error parsing the JSON response.
     """
-    api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     url = f"{BASE_URL}/models"
     if provider:
         kw = dict(headers=headers, params={"provider": provider})
     else:
         kw = dict(headers=headers)
-    response = _requests.get(url, **kw)
+    response = http.get(url, **kw)
     if response.status_code != 200:
         raise Exception(response.json())
     return _res_to_list(response)
@@ -104,24 +96,20 @@ def list_endpoints(
         BadRequestError: If there was an HTTP error.
         ValueError: If there was an error parsing the JSON response.
     """
-    api_key = _validate_api_key(api_key)
-    headers = {
-        "accept": "application/json",
-        "Authorization": f"Bearer {api_key}",
-    }
+    headers = _create_request_header(api_key)
     url = f"{BASE_URL}/endpoints"
     if model and provider:
         raise ValueError("Please specify either model OR provider, not both.")
     elif model:
         kw = dict(headers=headers, params={"model": model})
         return _res_to_list(
-            _requests.get(url, headers=headers, params={"model": model}),
+            http.get(url, headers=headers, params={"model": model}),
         )
     elif provider:
         kw = dict(headers=headers, params={"provider": provider})
     else:
         kw = dict(headers=headers)
-    response = _requests.get(url, **kw)
+    response = http.get(url, **kw)
     if response.status_code != 200:
         raise Exception(response.json())
     return _res_to_list(response)
