@@ -2,6 +2,7 @@ import inspect
 import json
 import os
 import threading
+import warnings
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 import requests
@@ -11,22 +12,6 @@ from pydantic import BaseModel, ValidationError
 PROJECT_LOCK = threading.Lock()
 
 
-class RequestError(Exception):
-    def __init__(self, response: requests.Response):
-        req = response.request
-        message = (
-            f"{req.method} {req.url} failed with status code {response.status_code}. "
-            f"Request body: {req.body}, Response: {response.text}"
-        )
-        super().__init__(message)
-        self.response = response
-
-
-def _check_response(response: requests.Response):
-    if not response.ok:
-        raise RequestError(response)
-
-
 def _res_to_list(response: requests.Response) -> Union[List, Dict]:
     return json.loads(response.text)
 
@@ -41,6 +26,27 @@ def _validate_api_key(api_key: Optional[str]) -> str:
     return api_key
 
 
+def _create_request_header(api_key: Optional[str]) -> Dict[str, str]:
+    return {
+        "Authorization": f"Bearer {_validate_api_key(api_key)}",
+        "accept": "application/json",
+        "Content-Type": "application/json",
+    }
+
+
+def _validate_openai_api_key(direct_mode: bool, api_key: Optional[str]) -> str:
+    if not direct_mode:
+        return None
+    if api_key is None:
+        api_key = os.environ.get("OPENAI_API_KEY")
+    if api_key is None:
+        warnings.warn(
+            "OPENAI_API_KEY is missing when trying to use direct mode. "
+            "Falling back to Unify API.",
+        )
+    return api_key
+
+
 def _default(value: Any, default_value: Any) -> Any:
     return value if value is not None else default_value
 
